//////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2003-2010 Daniel Vik
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef CRC32_H
#define CRC32_H

#include <list>
#include <stack>

typedef unsigned char UInt8;
typedef unsigned int UInt32;

class Crc32
{
public:
    Crc32();
    ~Crc32();

    // Calculate CRC tables based on a polynomial. (Default values for the
    // tables are provided in the implementation. They are based on the
    // polynomial 0x04c11db7, used in e.g. PKZip, WinZip and Ethernet.
    static void CalculateTables(UInt32 polynomial = 0x04c11db7);

    // Add data to CRC
    void append(UInt8 value);
    void append(const void* buffer, UInt32 size);

    // Rewind CRC
    void rewind(UInt8 value);
    void rewind(const void* buffer, UInt32 size);

    // Find CRC patch
    const UInt8* findPatch(UInt32 desiredCrc) const;

    // Find CRC patch with only ASCII characters
    const char* findPatchAscii(UInt32 desiredCrc) const;

    // Accessor methods
    void set(UInt32 crc);
    UInt32 get() const;

    // Calculate the CRC at the beginning of a buffer based on CRC at the end.
    static std::list<UInt32> Rewind(void* buffer, UInt32 size, UInt32 crc);

private:
    struct CrcMatch {
        UInt8* ptr;
        UInt32 offset;
        UInt32 crc;

        CrcMatch(void* _ptr, UInt32 _offset, UInt32 _crc) :
            ptr((UInt8*)_ptr), offset(_offset), crc(_crc) {}
    };

    static UInt32 Reflect(UInt32 value, int count);

    static UInt32 crc32Polynomial;

    static UInt32 crc32Table[256];

    static int patchCrc32Table[256];

    static UInt8 patchBytes[4];
    static char patchChars[7];

    UInt32 crc_;

    // Not implemented
    void operator=(const Crc32&);
    Crc32(const Crc32&);
};


#endif /*CRC_H*/
